//////////////////////////////////////////////////////////////////////////
// Mapper112  Nintendo MMC3                                             //
//////////////////////////////////////////////////////////////////////////

void	Mapper112::Reset()
{
	SetPROM_32K_Bank( 0, 1, PROM_8K_SIZE-2, PROM_8K_SIZE-1 );

	if( VROM_1K_SIZE ) {
		SetVROM_8K_Bank( 0 );
	}

	reg = 0;
	chr01 = 0;
	chr23 = 2;
	chr4  = 4;
	chr5  = 5;
	chr6  = 6;
	chr7  = 7;
}

void	Mapper112::Write( WORD addr, BYTE data )
{
	switch( addr ) {
		case	0x8000:
			reg = data;
			SetBank_PPU();
			break;
		case	0xA000:
			switch( reg & 0x07 ) {
				case	0x02:
					chr01 = data & 0xFE;
					SetBank_PPU();
					break;
				case	0x03:
					chr23 = data & 0xFE;
					SetBank_PPU();
					break;
				case	0x04:
					chr4 = data;
					SetBank_PPU();
					break;
				case	0x05:
					chr5 = data;
					SetBank_PPU();
					break;
				case	0x06:
					chr6 = data;
					SetBank_PPU();
					break;
				case	0x07:
					chr7 = data;
					SetBank_PPU();
					break;
				case	0x0:
					data = data & (PROM_8K_SIZE*2-1);
					if( reg&0x40 ) {
						SetPROM_8K_Bank( 6, data );
					} else {
						SetPROM_8K_Bank( 4, data );
					}
					break;
				case	0x01:
					data = data & (PROM_8K_SIZE*2-1);
					if( reg & 0x40 ) {
						SetPROM_8K_Bank( 5, data );
						SetPROM_8K_Bank( 4, ((PROM_8K_SIZE-1)*2) );
					} else {
						SetPROM_8K_Bank( 5, data );
						SetPROM_8K_Bank( 6, ((PROM_8K_SIZE-1)*2) );
					}
					break;
			}
			break;
		case	0xE000:
			if( !nes->rom->Is4SCREEN() ) {
				if( data & 0x01 ) SetVRAM_Mirror( VRAM_HMIRROR );
				else		  SetVRAM_Mirror( VRAM_VMIRROR );
			}
			break;
	}
}

void	Mapper112::SetBank_PPU()
{
	if( VROM_1K_SIZE ) {
		SetVROM_8K_Bank( chr01, chr01+1, chr23, chr23+1,
				 chr4, chr5, chr6, chr7 );
	}
}

void	Mapper112::SaveState( LPBYTE p )
{
	p[ 0] = reg;
	p[ 1] = chr01;
	p[ 2] = chr23;
	p[ 3] = chr4;
	p[ 4] = chr5;
	p[ 5] = chr6;
	p[ 6] = chr7;
}

void	Mapper112::LoadState( LPBYTE p )
{
	reg   = p[ 0];
	chr01 = p[ 1];
	chr23 = p[ 2];
	chr4  = p[ 3];
	chr5  = p[ 4];
	chr6  = p[ 5];
	chr7  = p[ 6];
}
